@@ -7,7 +7,9 @@ package gowid
 import (
 	"fmt"
 	"os"
+	"path/filepath"
 	"runtime/debug"
+	"strings"
 	"sync"
 	"time"
 
@@ -94,6 +96,8 @@ type App struct {
 	copyClaimedBy     IIdentity
 	copyLevel         int
 	refreshCopy       bool
+	prevWasMouseMove  bool // True if we last processed simple mouse movement. We can optimize on slow
+	// systems by discarding subsequent mouse movement events.
 
 	lastMouse    MouseState    // So I can tell if a button was previously clicked
 	MouseState                 // Track which mouse buttons are currently down
@@ -101,11 +105,14 @@ type App struct {
 	log          log.StdLogger // For any application logging
 }
 
+var _ IApp = (*App)(nil)
+
 // AppArgs is a helper struct, providing arguments for the initialization of App.
 type AppArgs struct {
-	View    IWidget
-	Palette IPalette
-	Log     log.StdLogger
+	View         IWidget
+	Palette      IPalette
+	Log          log.StdLogger
+	DontActivate bool
 }
 
 // IUnhandledInput is used as a handler for application user input that is not handled by any
@@ -215,19 +222,23 @@ func (m MouseState) RightIsClicked() bool {
 
 //======================================================================
 
+func NewApp(args AppArgs) (rapp *App, rerr error) {
+	app, err := newApp(args)
+	if err != nil {
+		return nil, err
+	}
+	return app, nil
+}
+
 // NewAppSafe returns an initialized App struct, or an error on failure. It will
 // initialize a tcell.Screen object behind the scenes, and enable mouse support
 // meaning that tcell will receive mouse events if the terminal supports them.
-func NewApp(args AppArgs) (rapp *App, rerr error) {
+func newApp(args AppArgs) (rapp *App, rerr error) {
 	screen, err := tcell.NewScreen()
 	if err != nil {
 		rerr = WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
 		return
 	}
-	if err := screen.Init(); err != nil {
-		rerr = WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
-		return
-	}
 
 	var palette IPalette = args.Palette
 	if palette == nil {
@@ -240,7 +251,15 @@ func NewApp(args AppArgs) (rapp *App, rerr error) {
 	clicks := MakeClickTargets()
 
 	if args.Log == nil {
-		args.Log = log.New()
+		logname := filepath.Base(os.Args[0])
+		logname = fmt.Sprintf("%s.log", strings.TrimSuffix(logname, filepath.Ext(logname)))
+		logfile, err := os.Create(logname)
+		if err != nil {
+			return nil, err
+		}
+		logger := log.New()
+		logger.Out = logfile
+		args.Log = logger
 	}
 
 	res := &App{
@@ -256,21 +275,12 @@ func NewApp(args AppArgs) (rapp *App, rerr error) {
 		log:               args.Log,
 	}
 
-	defFg := ColorDefault
-	defBg := ColorDefault
-	defSt := StyleNone
-	if paletteDefault, ok := res.IPalette.CellStyler("default"); ok {
-		fgCol, bgCol, style := paletteDefault.GetStyle(res)
-		defFg = IColorToTCell(fgCol, defFg, res.GetColorMode())
-		defBg = IColorToTCell(bgCol, defBg, res.GetColorMode())
-		defSt = defSt.MergeUnder(style)
+	if !args.DontActivate {
+		if err = res.initScreen(); err != nil {
+			return nil, err
+		}
 	}
-	defStyle := tcell.Style(defSt.OnOff).Background(defBg.ToTCell()).Foreground(defFg.ToTCell())
-	// Ask TCell to set the screen's default style according to the palette's "default"
-	// config, if one is provided. This might make every screen cell underlined, for example,
-	// in the absence of overriding styling from widgets.
-	screen.SetStyle(defStyle)
-	screen.EnableMouse()
+
 	screen.Clear()
 
 	cols := screen.Colors()
@@ -452,27 +462,29 @@ func (a *App) HandleTCellEvent(ev interface{}, unhandled IUnhandledInput) {
 		}
 		a.RedrawTerminal()
 	case *tcell.EventMouse:
-		switch ev.Buttons() {
-		case tcell.Button1:
-			a.MouseLeftClicked = true
-		case tcell.Button2:
-			a.MouseMiddleClicked = true
-		case tcell.Button3:
-			a.MouseRightClicked = true
-		default:
+		if !a.prevWasMouseMove || ev.Modifiers() != 0 || ev.Buttons() != 0 {
+			switch ev.Buttons() {
+			case tcell.Button1:
+				a.MouseLeftClicked = true
+			case tcell.Button2:
+				a.MouseMiddleClicked = true
+			case tcell.Button3:
+				a.MouseRightClicked = true
+			default:
+			}
+			debug.SetGCPercent(-1)
+			defer debug.SetGCPercent(100)
+			a.handleInputEvent(ev, unhandled)
+			// Make sure we don't hold on to references longer than we need to
+			if ev.Buttons() == tcell.ButtonNone {
+				a.ClickTargets.DeleteClickTargets(tcell.Button1)
+				a.ClickTargets.DeleteClickTargets(tcell.Button2)
+				a.ClickTargets.DeleteClickTargets(tcell.Button3)
+			}
+			a.lastMouse = a.MouseState
+			a.MouseState = MouseState{}
+			a.RedrawTerminal()
 		}
-		debug.SetGCPercent(-1)
-		defer debug.SetGCPercent(100)
-		a.handleInputEvent(ev, unhandled)
-		// Make sure we don't hold on to references longer than we need to
-		if ev.Buttons() == tcell.ButtonNone {
-			a.ClickTargets.DeleteClickTargets(tcell.Button1)
-			a.ClickTargets.DeleteClickTargets(tcell.Button2)
-			a.ClickTargets.DeleteClickTargets(tcell.Button3)
-		}
-		a.lastMouse = a.MouseState
-		a.MouseState = MouseState{}
-		a.RedrawTerminal()
 	case *tcell.EventResize:
 		if flog, ok := a.log.(log.FieldLogger); ok {
 			flog.WithField("event", ev).Infof("Terminal was resized")
@@ -499,6 +511,12 @@ func (a *App) HandleTCellEvent(ev interface{}, unhandled IUnhandledInput) {
 			a.log.Printf("Unanticipated event from tcell: %v\n", ev)
 		}
 	}
+
+	if ev, ok := ev.(*tcell.EventMouse); ok && ev.Buttons() == 0 && ev.Modifiers() == 0 {
+		a.prevWasMouseMove = true
+	} else {
+		a.prevWasMouseMove = false
+	}
 }
 
 // Close should be called by a gowid application after the user terminates the application.
@@ -752,6 +770,51 @@ func (a *App) Quit() {
 	close(a.AfterRenderEvents)
 }
 
+// Let screen be taken over by gowid/tcell. A new screen struct is created because
+// I can't make tcell claim and release the same screen successfully. Clients of
+// the app struct shouldn't cache the screen object returned via GetScreen().
+//
+func (a *App) ActivateScreen() error {
+	screen, err := tcell.NewScreen()
+	if err != nil {
+		return WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
+	}
+	a.screen = screen
+	if err := a.initScreen(); err != nil {
+		return err
+	}
+	return nil
+}
+
+func (a *App) DeactivateScreen() {
+	a.screen.Fini()
+	a.screen = nil
+}
+
+func (a *App) initScreen() error {
+	if err := a.screen.Init(); err != nil {
+		return WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
+	}
+
+	defFg := ColorDefault
+	defBg := ColorDefault
+	defSt := StyleNone
+	if paletteDefault, ok := a.IPalette.CellStyler("default"); ok {
+		fgCol, bgCol, style := paletteDefault.GetStyle(a)
+		defFg = IColorToTCell(fgCol, defFg, a.GetColorMode())
+		defBg = IColorToTCell(bgCol, defBg, a.GetColorMode())
+		defSt = defSt.MergeUnder(style)
+	}
+	defStyle := tcell.Style(defSt.OnOff).Background(defBg.ToTCell()).Foreground(defFg.ToTCell())
+	// Ask TCell to set the screen's default style according to the palette's "default"
+	// config, if one is provided. This might make every screen cell underlined, for example,
+	// in the absence of overriding styling from widgets.
+	a.screen.SetStyle(defStyle)
+	a.screen.EnableMouse()
+
+	return nil
+}
+
 //======================================================================
 // Local Variables:
 // mode: Go
