@@ -10,6 +10,8 @@ import (
 
 	"github.com/gcla/gowid/gwutil"
 	"github.com/gdamore/tcell"
+	lru "github.com/hashicorp/golang-lru"
+	"github.com/lucasb-eyer/go-colorful"
 	"github.com/pkg/errors"
 )
 
@@ -134,11 +136,16 @@ func (c ColorMode) String() string {
 const (
 	colorDefaultName      = "default"
 	colorBlackName        = "black"
+	colorRedName          = "red"
 	colorDarkRedName      = "dark red"
+	colorGreenName        = "green"
 	colorDarkGreenName    = "dark green"
 	colorBrownName        = "brown"
+	colorBlueName         = "blue"
 	colorDarkBlueName     = "dark blue"
+	colorMagentaName      = "magenta"
 	colorDarkMagentaName  = "dark magenta"
+	colorCyanName         = "cyan"
 	colorDarkCyanName     = "dark cyan"
 	colorLightGrayName    = "light gray"
 	colorDarkGrayName     = "dark gray"
@@ -149,57 +156,59 @@ const (
 	colorLightMagentaName = "light magenta"
 	colorLightCyanName    = "light cyan"
 	colorWhiteName        = "white"
-
-	tColorDefaultName = "default"
-	tColorBlackName   = "black"
-	tColorRedName     = "red"
-	tColorGreenName   = "green"
-	tColorYellowName  = "yellow"
-	tColorBlueName    = "blue"
-	tColorMagentaName = "magenta"
-	tColorCyanName    = "cyan"
-	tColorWhiteName   = "white"
 )
 
 var (
-	basicColors = []string{
-		colorDefaultName,
-		colorBlackName,
-		colorDarkRedName,
-		colorDarkGreenName,
-		colorBrownName,
-		colorDarkBlueName,
-		colorDarkMagentaName,
-		colorDarkCyanName,
-		colorLightGrayName,
-		colorDarkGrayName,
-		colorLightRedName,
-		colorLightGreenName,
-		colorYellowName,
-		colorLightBlueName,
-		colorLightMagentaName,
-		colorLightCyanName,
-		colorWhiteName,
+	basicColors = map[string]int{
+		colorDefaultName:      0,
+		colorBlackName:        1,
+		colorDarkRedName:      2,
+		colorDarkGreenName:    3,
+		colorBrownName:        4,
+		colorDarkBlueName:     5,
+		colorDarkMagentaName:  6,
+		colorDarkCyanName:     7,
+		colorLightGrayName:    8,
+		colorDarkGrayName:     9,
+		colorLightRedName:     10,
+		colorLightGreenName:   11,
+		colorYellowName:       12,
+		colorLightBlueName:    13,
+		colorLightMagentaName: 14,
+		colorLightCyanName:    15,
+		colorWhiteName:        16,
+		colorRedName:          10,
+		colorGreenName:        11,
+		colorBlueName:         13,
+		colorMagentaName:      14,
+		colorCyanName:         15,
 	}
 
-	basicColorsMap map[string]int
-
-	tCellBasicColors = []string{
-		tColorDefaultName,
-		tColorBlackName,
-		tColorRedName,
-		tColorGreenName,
-		tColorYellowName,
-		tColorBlueName,
-		tColorMagentaName,
-		tColorCyanName,
-		tColorWhiteName,
+	tBasicColors = map[string]int{
+		colorDefaultName:      0,
+		colorBlackName:        1,
+		colorDarkRedName:      2,
+		colorDarkGreenName:    3,
+		colorBrownName:        4,
+		colorDarkBlueName:     5,
+		colorDarkMagentaName:  6,
+		colorDarkCyanName:     7,
+		colorLightGrayName:    8,
+		colorDarkGrayName:     1,
+		colorLightRedName:     2,
+		colorLightGreenName:   3,
+		colorYellowName:       4,
+		colorLightBlueName:    5,
+		colorLightMagentaName: 6,
+		colorLightCyanName:    7,
+		colorWhiteName:        8,
+		colorRedName:          2,
+		colorGreenName:        3,
+		colorBlueName:         5,
+		colorMagentaName:      6,
+		colorCyanName:         7,
 	}
 
-	NumOfTCellBasicColors = len(tCellBasicColors)
-
-	tBasicColorsMap map[string]int
-
 	CubeStart    = 16 // first index of color cube
 	CubeSize256  = 6  // one side of the color cube
 	graySize256  = 24
@@ -263,6 +272,98 @@ var (
 	shortColorRE   = regexp.MustCompile(`^#([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])$`)
 	grayHexColorRE = regexp.MustCompile(`^g#([0-9a-fA-F][0-9a-fA-F])$`)
 	grayDecColorRE = regexp.MustCompile(`^g(1?[0-9][0-9]?)$`)
+
+	colorfulBlack8   = colorful.Color{R: 0.0, G: 0.0, B: 0.0}
+	colorfulWhite8   = colorful.Color{R: 1.0, G: 1.0, B: 1.0}
+	colorfulRed8     = colorful.Color{R: 1.0, G: 0.0, B: 0.0}
+	colorfulGreen8   = colorful.Color{R: 0.0, G: 1.0, B: 0.0}
+	colorfulBlue8    = colorful.Color{R: 0.0, G: 0.0, B: 1.0}
+	colorfulYellow8  = colorful.Color{R: 1.0, G: 1.0, B: 0.0}
+	colorfulMagenta8 = colorful.Color{R: 1.0, G: 0.0, B: 1.0}
+	colorfulCyan8    = colorful.Color{R: 0.0, G: 1.0, B: 1.0}
+
+	colorfulBlack16         = colorful.Color{R: 0.0, G: 0.0, B: 0.0}
+	colorfulWhite16         = colorful.Color{R: 0.66, G: 0.66, B: 0.66}
+	colorfulRed16           = colorful.Color{R: 0.5, G: 0.0, B: 0.0}
+	colorfulGreen16         = colorful.Color{R: 0.0, G: 0.5, B: 0.0}
+	colorfulBlue16          = colorful.Color{R: 0.0, G: 0.0, B: 0.5}
+	colorfulYellow16        = colorful.Color{R: 0.5, G: 0.5, B: 0.5}
+	colorfulMagenta16       = colorful.Color{R: 0.5, G: 0.0, B: 0.5}
+	colorfulCyan16          = colorful.Color{R: 0.0, G: 0.5, B: 0.5}
+	colorfulBrightBlack16   = colorful.Color{R: 0.33, G: 0.33, B: 0.33}
+	colorfulBrightWhite16   = colorful.Color{R: 1.0, G: 1.0, B: 1.0}
+	colorfulBrightRed16     = colorful.Color{R: 1.0, G: 0.0, B: 0.0}
+	colorfulBrightGreen16   = colorful.Color{R: 0.0, G: 1.0, B: 0.0}
+	colorfulBrightBlue16    = colorful.Color{R: 0.0, G: 0.0, B: 1.0}
+	colorfulBrightYellow16  = colorful.Color{R: 1.0, G: 1.0, B: 1.0}
+	colorfulBrightMagenta16 = colorful.Color{R: 1.0, G: 0.0, B: 1.0}
+	colorfulBrightCyan16    = colorful.Color{R: 0.0, G: 1.0, B: 1.0}
+
+	// Used in mapping RGB colors down to 8 terminal colors.
+	colorful8 = []colorful.Color{
+		colorfulBlack8,
+		colorfulWhite8,
+		colorfulRed8,
+		colorfulGreen8,
+		colorfulBlue8,
+		colorfulYellow8,
+		colorfulMagenta8,
+		colorfulCyan8,
+	}
+
+	// Used in mapping RGB colors down to 16 terminal colors.
+	colorful16 = []colorful.Color{
+		colorfulBlack16,
+		colorfulWhite16,
+		colorfulRed16,
+		colorfulGreen16,
+		colorfulBlue16,
+		colorfulYellow16,
+		colorfulMagenta16,
+		colorfulCyan16,
+		colorfulBrightBlack16,
+		colorfulBrightWhite16,
+		colorfulBrightRed16,
+		colorfulBrightGreen16,
+		colorfulBrightBlue16,
+		colorfulBrightYellow16,
+		colorfulBrightMagenta16,
+		colorfulBrightCyan16,
+	}
+
+	term8 = []TCellColor{
+		ColorBlack,
+		ColorWhite,
+		ColorRed,
+		ColorGreen,
+		ColorBlue,
+		ColorYellow,
+		ColorMagenta,
+		ColorCyan,
+	}
+
+	term16 = []TCellColor{
+		ColorBlack,
+		ColorLightGray,
+		ColorDarkRed,
+		ColorDarkGreen,
+		ColorDarkBlue,
+		ColorYellow,
+		ColorMagenta,
+		ColorCyan,
+		ColorDarkGray,
+		ColorWhite,
+		ColorRed,
+		ColorGreen,
+		ColorBlue,
+		ColorYellow,
+		ColorMagenta,
+		ColorCyan, // TODO - figure out these colors
+	}
+
+	term2Cache  *lru.Cache
+	term8Cache  *lru.Cache
+	term16Cache *lru.Cache
 )
 
 //======================================================================
@@ -281,13 +382,13 @@ func init() {
 		grayLookup256_101[i] = grayLookup256[intScale(i, 101, 0x100)]
 		grayLookup88_101[i] = grayLookup88[intScale(i, 101, 0x100)]
 	}
-	basicColorsMap = make(map[string]int)
-	tBasicColorsMap = make(map[string]int)
-	for i, v := range basicColors {
-		basicColorsMap[v] = i
-	}
-	for i, v := range tCellBasicColors {
-		tBasicColorsMap[v] = i
+
+	var err error
+	for _, cache := range []**lru.Cache{&term2Cache, &term8Cache, &term16Cache} {
+		*cache, err = lru.New(100)
+		if err != nil {
+			panic(err)
+		}
 	}
 }
 
@@ -537,6 +638,38 @@ func MakeRGBColorExt(r, g, b int) RGBColor {
 	return res
 }
 
+// Implements golang standard library's color.Color
+func (rgb RGBColor) RGBA() (r, g, b, a uint32) {
+	r = uint32(rgb.Red << 8)
+	g = uint32(rgb.Green << 8)
+	b = uint32(rgb.Blue << 8)
+	a = 0xffff
+	return
+}
+
+func (r RGBColor) findClosest(from []colorful.Color, corresponding []TCellColor, cache *lru.Cache) TCellColor {
+	var best float64 = 100.0
+	var j int
+
+	if res, ok := cache.Get(r); ok {
+		return res.(TCellColor)
+	}
+
+	ccol, _ := colorful.MakeColor(r)
+
+	for i, c := range from {
+		x := c.DistanceLab(ccol)
+		if x < best {
+			best = x
+			j = i
+		}
+	}
+
+	cache.Add(r, corresponding[j])
+
+	return corresponding[j]
+}
+
 // ToTCellColor converts an RGBColor to a TCellColor, suitable for rendering to the screen
 // with tcell. It lets RGBColor conform to IColor.
 func (r RGBColor) ToTCellColor(mode ColorMode) (TCellColor, bool) {
@@ -556,8 +689,14 @@ func (r RGBColor) ToTCellColor(mode ColorMode) (TCellColor, bool) {
 		b := cubeLookup88_16[r.Blue>>4]
 		c := tcell.Color((CubeStart + (((rd * cubeSize88) + g) * cubeSize88) + b) + 0)
 		return MakeTCellColorExt(c), true
+	case Mode16Colors:
+		return r.findClosest(colorful16, term16, term16Cache), true
+	case Mode8Colors:
+		return r.findClosest(colorful8, term8, term8Cache), true
+	case ModeMonochrome:
+		return r.findClosest(colorful8[0:1], term8[0:1], term2Cache), true
 	default:
-		panic(errors.WithStack(ColorModeMismatch{Color: r, Mode: mode}))
+		return TCellColor{}, false
 	}
 }
 
@@ -616,9 +755,9 @@ func (s *UrwidColor) ToTCellColor(mode ColorMode) (TCellColor, bool) {
 	idx := -1
 	switch mode {
 	case Mode24BitColors, Mode256Colors, Mode88Colors, Mode16Colors:
-		idx = posInMap(s.Id, basicColorsMap)
+		idx = posInMap(s.Id, basicColors)
 	case Mode8Colors, ModeMonochrome:
-		idx = posInMap(s.Id, tBasicColorsMap)
+		idx = posInMap(s.Id, tBasicColors)
 	default:
 		panic(errors.WithStack(ColorModeMismatch{Color: s, Mode: mode}))
 	}
