@@ -421,7 +421,7 @@ type ISelectChild interface {
 // focus - it might be given focus if there is no other option (no other
 // selectable widgets in the container, for example).
 //
-// UserEvent() is provided the TCell event (mouse or keyboard action),
+// UserInput() is provided the TCell event (mouse or keyboard action),
 // the size spec that would be given to Render(), whether or not the widget
 // has focus, and access to the application, useful for effecting changes
 // like changing colors, running a function, or quitting. The render size is
@@ -659,6 +659,10 @@ type IClipboard interface {
 	Clips(app IApp) []ICopyResult
 }
 
+// IClipboardSelected is implemented by widgets that support changing their
+// look when they have been "selected" in some application-level copy-mode,
+// the idea being to provide the user with the information that this widget's
+// contents will be copied.
 type IClipboardSelected interface {
 	AlterWidget(w IWidget, app IApp) IWidget
 }
@@ -771,48 +775,24 @@ func CalculateRenderSizeFallback(w IWidget, size IRenderSize, focus Selector, ap
 		res.R = sz.BoxRows()
 		res.C = sz.BoxColumns()
 	default:
-		c := Render(w, size, focus, app)
+		c := w.Render(size, focus, app)
 		res.R = c.BoxRows()
 		res.C = c.BoxColumns()
 	}
 	return res
 }
 
-// Render currently passes control through to the widget's Render method. Having
-// this function allows for easier instrumentation of the Render path. The function
-// returns a canvas representing the rendered widget.
-func Render(w IWidget, size IRenderSize, focus Selector, app IApp) ICanvas {
-	res := w.Render(size, focus, app)
-
-	// Enable when debugging
-	// PanicIfCanvasNotRightSize(res, size)
-
-	return res
-}
-
 // UserInputIfSelectable will return false if the widget is not selectable; otherwise it will
 // try the widget's UserInput function.
 func UserInputIfSelectable(w IWidget, ev interface{}, size IRenderSize, focus Selector, app IApp) bool {
 	res := false
 	if w.Selectable() {
-		res = UserInput(w, ev, size, focus, app)
+		res = w.UserInput(ev, size, focus, app)
 	}
 
 	return res
 }
 
-// UserInput currently passes control through to the widget's UserInput method. Having
-// this function allows for easier instrumentation of the UserInput path. UserInput
-// should return true if this widget "handles" the provided input, and false
-// otherwise. This return value can guide parent widgets and help them determine
-// whether or not they should then consume the input event. Note that returning
-// true does not guarantee no other widget will also handle the event - for example
-// the ListBox widget may handle a mouse click by changing the focus widget, but
-// also allow the child widget at focus to receive the click too.
-func UserInput(w IWidget, ev interface{}, size IRenderSize, focus Selector, app IApp) bool {
-	return w.UserInput(ev, size, focus, app)
-}
-
 // RenderSize currently passes control through to the widget's RenderSize
 // method. Having this function allows for easier instrumentation of the
 // RenderSize path. RenderSize is intended to compute the size of the canvas
@@ -839,7 +819,7 @@ func SubWidgetSize(w ICompositeWidget, size IRenderSize, focus Selector, app IAp
 // with an IRenderBox size argument equal to the size of the current terminal.
 func RenderRoot(w IWidget, t *App) {
 	maxX, maxY := t.TerminalSize()
-	canvas := Render(w, RenderBox{C: maxX, R: maxY}, Focused, t)
+	canvas := w.Render(RenderBox{C: maxX, R: maxY}, Focused, t)
 
 	// tcell will apply its default style to empty cells. But because gowid's model
 	// is to layer styles, here we explicitly merge each canvas cell on top of a cell
@@ -968,8 +948,10 @@ func (f WidgetCallback) ID() interface{} {
 }
 
 func RunWidgetCallbacks(c ICallbacks, name interface{}, app IApp, data ...interface{}) {
-	data2 := append([]interface{}{app}, data...)
-	c.RunCallbacks(name, data2...)
+	if c != nil {
+		data2 := append([]interface{}{app}, data...)
+		c.RunCallbacks(name, data2...)
+	}
 }
 
 type widgetChangedCallbackProxy struct {
@@ -978,7 +960,8 @@ type widgetChangedCallbackProxy struct {
 
 func (p widgetChangedCallbackProxy) Call(args ...interface{}) {
 	t := args[0].(IApp)
-	w := args[1].(IWidget)
+	var w IWidget
+	w, _ = args[1].(IWidget)
 	p.IWidgetChangedCallback.Changed(t, w, args[2:]...)
 }
 
@@ -1003,15 +986,18 @@ func QuitFn(app IApp, widget IWidget) {
 // SubWidgetCallbacks is a convenience struct for embedding in a widget, providing methods
 // to add and remove callbacks that are executed when the widget's child is modified.
 type SubWidgetCallbacks struct {
-	ICallbacks
+	CB **Callbacks
 }
 
 func (w *SubWidgetCallbacks) OnSetSubWidget(f IWidgetChangedCallback) {
-	AddWidgetCallback(w, SubWidgetCB{}, f)
+	if *w.CB == nil {
+		*w.CB = NewCallbacks()
+	}
+	AddWidgetCallback(*w.CB, SubWidgetCB{}, f)
 }
 
 func (w *SubWidgetCallbacks) RemoveOnSetSubWidget(f IIdentity) {
-	RemoveWidgetCallback(w, SubWidgetCB{}, f)
+	RemoveWidgetCallback(*w.CB, SubWidgetCB{}, f)
 }
 
 //======================================================================
@@ -1019,15 +1005,18 @@ func (w *SubWidgetCallbacks) RemoveOnSetSubWidget(f IIdentity) {
 // SubWidgetsCallbacks is a convenience struct for embedding in a widget, providing methods
 // to add and remove callbacks that are executed when the widget's children are modified.
 type SubWidgetsCallbacks struct {
-	ICallbacks
+	CB **Callbacks
 }
 
 func (w *SubWidgetsCallbacks) OnSetSubWidgets(f IWidgetChangedCallback) {
-	AddWidgetCallback(w, SubWidgetsCB{}, f)
+	if *w.CB == nil {
+		*w.CB = NewCallbacks()
+	}
+	AddWidgetCallback(*w.CB, SubWidgetsCB{}, f)
 }
 
 func (w *SubWidgetsCallbacks) RemoveOnSetSubWidgets(f IIdentity) {
-	RemoveWidgetCallback(w, SubWidgetsCB{}, f)
+	RemoveWidgetCallback(*w.CB, SubWidgetsCB{}, f)
 }
 
 //======================================================================
@@ -1035,15 +1024,18 @@ func (w *SubWidgetsCallbacks) RemoveOnSetSubWidgets(f IIdentity) {
 // ClickCallbacks is a convenience struct for embedding in a widget, providing methods
 // to add and remove callbacks that are executed when the widget is "clicked".
 type ClickCallbacks struct {
-	ICallbacks
+	CB **Callbacks
 }
 
 func (w *ClickCallbacks) OnClick(f IWidgetChangedCallback) {
-	AddWidgetCallback(w, ClickCB{}, f)
+	if *w.CB == nil {
+		*w.CB = NewCallbacks()
+	}
+	AddWidgetCallback(*w.CB, ClickCB{}, f)
 }
 
 func (w *ClickCallbacks) RemoveOnClick(f IIdentity) {
-	RemoveWidgetCallback(w, ClickCB{}, f)
+	RemoveWidgetCallback(*w.CB, ClickCB{}, f)
 }
 
 //======================================================================
@@ -1051,15 +1043,18 @@ func (w *ClickCallbacks) RemoveOnClick(f IIdentity) {
 // KeyPressCallbacks is a convenience struct for embedding in a widget, providing methods
 // to add and remove callbacks that are executed when the widget is "clicked".
 type KeyPressCallbacks struct {
-	ICallbacks
+	CB **Callbacks
 }
 
 func (w *KeyPressCallbacks) OnKeyPress(f IWidgetChangedCallback) {
-	AddWidgetCallback(w, KeyPressCB{}, f)
+	if *w.CB == nil {
+		*w.CB = NewCallbacks()
+	}
+	AddWidgetCallback(*w.CB, KeyPressCB{}, f)
 }
 
 func (w *KeyPressCallbacks) RemoveOnKeyPress(f IIdentity) {
-	RemoveWidgetCallback(w, KeyPressCB{}, f)
+	RemoveWidgetCallback(*w.CB, KeyPressCB{}, f)
 }
 
 //======================================================================
@@ -1067,15 +1062,18 @@ func (w *KeyPressCallbacks) RemoveOnKeyPress(f IIdentity) {
 // FocusCallbacks is a convenience struct for embedding in a widget, providing methods
 // to add and remove callbacks that are executed when the widget's focus widget changes.
 type FocusCallbacks struct {
-	ICallbacks
+	CB **Callbacks
 }
 
 func (w *FocusCallbacks) OnFocusChanged(f IWidgetChangedCallback) {
-	AddWidgetCallback(w, FocusCB{}, f)
+	if *w.CB == nil {
+		*w.CB = NewCallbacks()
+	}
+	AddWidgetCallback(*w.CB, FocusCB{}, f)
 }
 
 func (w *FocusCallbacks) RemoveOnFocusChanged(f IIdentity) {
-	RemoveWidgetCallback(w, FocusCB{}, f)
+	RemoveWidgetCallback(*w.CB, FocusCB{}, f)
 }
 
 //======================================================================
@@ -1677,7 +1675,7 @@ func CopyModeUserInput(w ICopyModeWidget, ev interface{}, size IRenderSize, focu
 		} else {
 			cl := app.CopyLevel()
 			app.CopyLevel(cl + lvls) // this is how many levels hexdumper will support
-			res = UserInput(w.SubWidget(), ev, size, focus, app)
+			res = w.SubWidget().UserInput(ev, size, focus, app)
 			app.CopyLevel(cl)
 
 			if !res {
@@ -1689,7 +1687,7 @@ func CopyModeUserInput(w ICopyModeWidget, ev interface{}, size IRenderSize, focu
 		evc.Action.Collect(w.Clips(app))
 		res = true
 	} else {
-		res = UserInput(w.SubWidget(), ev, size, focus, app)
+		res = w.SubWidget().UserInput(ev, size, focus, app)
 	}
 	return res
 }
