@@ -24,16 +24,18 @@ type IWidget interface {
 	gowid.IFindNextSelectable
 	gowid.IPreferedPosition
 	gowid.ISelectChild
+	gowid.IIdentity
 	WidgetWidths(size gowid.IRenderSize, focus gowid.Selector, focusIdx int, app gowid.IApp) []int
 	Wrap() bool
 }
 
 type Widget struct {
-	widgets   []gowid.IContainerWidget
-	focus     int // -1 means nothing selectable
-	prefCol   int // caches the last set prefered col. Passes it on if widget hasn't changed focus
-	opt       Options
-	Callbacks *gowid.Callbacks
+	widgets []gowid.IContainerWidget
+	focus   int // -1 means nothing selectable
+	prefCol int // caches the last set prefered col. Passes it on if widget hasn't changed focus
+	opt     Options
+	*gowid.Callbacks
+	gowid.AddressProvidesID
 	gowid.SubWidgetsCallbacks
 	gowid.FocusCallbacks
 }
@@ -53,16 +55,15 @@ func New(widgets []gowid.IContainerWidget, opts ...Options) *Widget {
 			StartColumn: -1,
 		}
 	}
-	cb := gowid.NewCallbacks()
 	res := &Widget{
-		widgets:             widgets,
-		focus:               -1,
-		prefCol:             -1,
-		opt:                 opt,
-		Callbacks:           cb,
-		SubWidgetsCallbacks: gowid.SubWidgetsCallbacks{cb},
-		FocusCallbacks:      gowid.FocusCallbacks{ICallbacks: cb},
+		widgets: widgets,
+		focus:   -1,
+		prefCol: -1,
+		opt:     opt,
 	}
+	res.SubWidgetsCallbacks = gowid.SubWidgetsCallbacks{CB: &res.Callbacks}
+	res.FocusCallbacks = gowid.FocusCallbacks{CB: &res.Callbacks}
+
 	if opt.StartColumn >= 0 {
 		res.focus = gwutil.Min(opt.StartColumn, len(widgets)-1)
 	} else {
@@ -257,7 +258,12 @@ func SubWidgetSize(size gowid.IRenderSize, newX int, dim gowid.IWidgetDimension)
 
 	switch sz := size.(type) {
 	case gowid.IRenderFixed:
-		subSize = gowid.RenderFixed{}
+		switch dim.(type) {
+		case gowid.IRenderBox:
+			subSize = dim
+		default:
+			subSize = gowid.RenderFixed{}
+		}
 	case gowid.IRenderBox:
 		switch dim.(type) {
 		case gowid.IRenderFixed:
@@ -300,12 +306,32 @@ func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Se
 		if evm, ok := ev.(*tcell.EventMouse); ok {
 			curX := 0
 			mx, _ := evm.Position()
+		Loop:
 			for i, c := range subSizes {
 				if mx < curX+c && mx >= curX {
 					subSize := w.SubWidgetSize(size, c, subs[i], dims[i])
-					forChild = gowid.UserInput(subs[i], gowid.TranslatedMouseEvent(ev, -curX, 0), subSize, focus.SelectIf(w.SelectChild(focus) && i == subfocus), app)
-					if forChild {
-						w.SetFocus(app, i)
+					forChild = subs[i].UserInput(gowid.TranslatedMouseEvent(ev, -curX, 0), subSize, focus.SelectIf(w.SelectChild(focus) && i == subfocus), app)
+
+					// Give the child focus if (a) it's selectable, and (b) if this is the click up corresponding
+					// to a previous click down on this columns widget.
+					switch evm.Buttons() {
+					case tcell.Button1, tcell.Button2, tcell.Button3:
+						app.SetClickTarget(evm.Buttons(), w)
+					case tcell.ButtonNone:
+						if !app.GetLastMouseState().NoButtonClicked() {
+							if subs[i].Selectable() {
+								clickit := false
+								app.ClickTarget(func(k tcell.ButtonMask, v gowid.IIdentityWidget) {
+									if v != nil && v.ID() == w.ID() {
+										clickit = true
+									}
+								})
+								if clickit {
+									w.SetFocus(app, i)
+								}
+							}
+						}
+						break Loop
 					}
 					break
 				}
@@ -407,6 +433,11 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 			trunc(&res[i])
 			colsUsed += res[i]
 			helper[i] = true
+		case gowid.IRenderBox:
+			res[i] = w2.BoxColumns()
+			trunc(&res[i])
+			colsUsed += res[i]
+			helper[i] = true
 		case gowid.IRenderFlowWith:
 			res[i] = w2.FlowColumns()
 			trunc(&res[i])
@@ -524,11 +555,11 @@ func Render(w IWidget, size gowid.IRenderSize, focus gowid.Selector, app gowid.I
 		diff := res.BoxRows() - canvases[i].BoxRows()
 		if diff > 0 {
 			fill := fill.NewEmpty()
-			fc := gowid.Render(fill, gowid.RenderBox{canvases[i].BoxColumns(), diff}, gowid.NotSelected, app)
+			fc := fill.Render(gowid.RenderBox{canvases[i].BoxColumns(), diff}, gowid.NotSelected, app)
 			canvases[i].AppendBelow(fc, false, false)
 		} else if diff < 0 {
 			fill := fill.NewEmpty()
-			fc := gowid.Render(fill, gowid.RenderBox{res.BoxColumns(), -diff}, gowid.NotSelected, app)
+			fc := fill.Render(gowid.RenderBox{res.BoxColumns(), -diff}, gowid.NotSelected, app)
 			res.AppendBelow(fc, false, false)
 		}
 		res.AppendRight(canvases[i], i == subfocus)
@@ -572,7 +603,7 @@ func RenderSubWidgets(w IWidget, size gowid.IRenderSize, focus gowid.Selector, f
 			maxes = append(maxes, i)
 			ssizes = append(ssizes, subSize)
 		} else {
-			canvases[i] = gowid.Render(subs[i], subSize, focus.SelectIf(w.SelectChild(focus) && i == focusIdx), app)
+			canvases[i] = subs[i].Render(subSize, focus.SelectIf(w.SelectChild(focus) && i == focusIdx), app)
 			if canvases[i].BoxRows() > curMax {
 				curMax = canvases[i].BoxRows()
 			}
@@ -593,7 +624,7 @@ func RenderSubWidgets(w IWidget, size gowid.IRenderSize, focus gowid.Selector, f
 			mss = gowid.MakeRenderBox(css.BoxColumns(), curMax)
 		default:
 		}
-		canvases[i] = gowid.Render(subs[i], mss, focus.SelectIf(w.SelectChild(focus) && i == focusIdx), app)
+		canvases[i] = subs[i].Render(mss, focus.SelectIf(w.SelectChild(focus) && i == focusIdx), app)
 	}
 
 	return canvases
