@@ -187,13 +187,17 @@ func (w *SimpleListWalker) Previous(ipos IWalkerPosition) IWalkerPosition {
 
 //======================================================================
 
-type IWidget interface {
-	gowid.IWidget
+type IListFns interface {
 	RenderSubwidgets(gowid.IRenderSize, gowid.Selector, gowid.IApp) ([]SubRenders, SubRenders, []SubRenders)
 	Walker() IWalker
 	SetWalker(IWalker, gowid.IApp)
 }
 
+type IWidget interface {
+	gowid.IWidget
+	IListFns
+}
+
 type Widget struct {
 	walker IWalker
 	// This says how many lines to cut from the top of the widget rendered at the top of the listbox.
@@ -201,6 +205,7 @@ type Widget struct {
 	st      state
 	options Options
 	gowid.AddressProvidesID
+	*gowid.Callbacks
 	gowid.FocusCallbacks
 	gowid.IsSelectable
 }
@@ -215,7 +220,7 @@ type IndexedWidget struct {
 }
 
 type state struct {
-	linesOffTop           int
+	linesOffTop           int // used only if focus widget has more lines than can be displayed
 	topToBottomRatio      float32
 	topToBottomRatioValid bool // Means denominator is 0 if true i.e. at the bottom
 }
@@ -226,10 +231,10 @@ func New(walker IWalker, opts ...Options) *Widget {
 		opt = opts[0]
 	}
 	res := &Widget{
-		walker:         walker,
-		options:        opt,
-		FocusCallbacks: gowid.FocusCallbacks{ICallbacks: gowid.NewCallbacks()},
+		walker:  walker,
+		options: opt,
 	}
+	res.FocusCallbacks = gowid.FocusCallbacks{CB: &res.Callbacks}
 	res.goToTop()
 
 	var _ gowid.IWidget = res
@@ -267,6 +272,18 @@ func (w *IndexedWidget) SetWalker(l IWalker, app gowid.IApp) {
 	w.Widget.SetWalker(l, app)
 }
 
+func (w *Widget) State() interface{} {
+	return w.st
+}
+
+func (w *Widget) SetState(st interface{}, app gowid.IApp) {
+	if state, ok := st.(state); !ok {
+		panic(BadState)
+	} else {
+		w.st = state
+	}
+}
+
 func (w *Widget) GoToTop(app gowid.IApp) {
 	w.goToTop()
 }
@@ -360,10 +377,10 @@ func (w *Widget) RenderSubwidgets(size gowid.IRenderSize, focus gowid.Selector,
 		var curToRender gowid.IWidget = curWidget
 		if haveCols {
 			//c = gowid.Render(curWidget, gowid.RenderFlowWith{C: cols.Columns()}, focus, app)
-			c = gowid.Render(curToRender, gowid.RenderFlowWith{C: cols.Columns()}, focus, app)
+			c = curToRender.Render(gowid.RenderFlowWith{C: cols.Columns()}, focus, app)
 		} else {
 			//c = gowid.Render(curWidget, gowid.RenderFixed{}, focus, app)
-			c = gowid.Render(curToRender, gowid.RenderFixed{}, focus, app)
+			c = curToRender.Render(gowid.RenderFixed{}, focus, app)
 		}
 		creallines := c.BoxRows()
 		middle = SubRenders{curWidget, curPos, c, creallines}
@@ -412,15 +429,14 @@ func (w *Widget) RenderSubwidgets(size gowid.IRenderSize, focus gowid.Selector,
 				} else {
 					var upC gowid.ICanvas
 					if haveCols {
-						upC = gowid.Render(upWidget, gowid.RenderFlowWith{C: cols.Columns()}, gowid.NotSelected, app)
+						upC = upWidget.Render(gowid.RenderFlowWith{C: cols.Columns()}, gowid.NotSelected, app)
 					} else {
-						upC = gowid.Render(upWidget, gowid.RenderFixed{}, gowid.NotSelected, app)
+						upC = upWidget.Render(gowid.RenderFixed{}, gowid.NotSelected, app)
 					}
 					upreallines := upC.BoxRows()
 					if haveLinesNeeded {
 						if upreallines > topLinesNeeded {
 							upC.Truncate(upreallines-topLinesNeeded, 0)
-							w.st.linesOffTop = upreallines - topLinesNeeded
 						}
 						topLinesNeeded -= upC.BoxRows()
 					}
@@ -440,9 +456,9 @@ func (w *Widget) RenderSubwidgets(size gowid.IRenderSize, focus gowid.Selector,
 				} else {
 					var downC gowid.ICanvas
 					if haveCols {
-						downC = gowid.Render(downWidget, gowid.RenderFlowWith{C: cols.Columns()}, gowid.NotSelected, app)
+						downC = downWidget.Render(gowid.RenderFlowWith{C: cols.Columns()}, gowid.NotSelected, app)
 					} else {
-						downC = gowid.Render(downWidget, gowid.RenderFixed{}, gowid.NotSelected, app)
+						downC = downWidget.Render(gowid.RenderFixed{}, gowid.NotSelected, app)
 					}
 					downreallines := downC.BoxRows()
 					if haveLinesNeeded {
@@ -470,7 +486,7 @@ func SubWidgetSize(w IWidget, size gowid.IRenderSize, focus gowid.Selector, app
 	}
 }
 
-func CalculateOnScreen(w IWidget, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) (int, int, int, error) {
+func CalculateOnScreen(w IListFns, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) (int, int, int, error) {
 	aboveMiddle, middle, belowMiddle := w.RenderSubwidgets(size, focus, app)
 	mc := 0
 	tc := 0
@@ -638,6 +654,7 @@ func (w *Widget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.S
 	}
 
 	forChild := false
+	childSelectable := false
 	curi := w.Walker().Focus()
 	position := curi
 	cur := w.Walker().At(position)
@@ -677,6 +694,7 @@ func (w *Widget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.S
 				}
 				sizeForInput := userInputSize()
 				forChild = gowid.UserInputIfSelectable(widgetRender.Widget, gowid.TranslatedMouseEvent(ev, 0, -curY), sizeForInput, gowid.Focused, app)
+				childSelectable = widgetRender.Widget.Selectable()
 				break
 			}
 			curY += widgetRender.Canvas.BoxRows()
@@ -747,45 +765,49 @@ func (w *Widget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.S
 			app.SetClickTarget(ev2.Buttons(), w)
 			res = true
 		case tcell.ButtonNone:
-			// tcell will report ButtonNone for mouse events which are simply pointer movements
-			// (at least in my terminal). To distinguish this from a mouse release event, we track
-			// the prior input's mouse state. If the last state was a mouse click, then this event
-			// is processed as a mouse button release.
-			if !app.GetLastMouseState().NoButtonClicked() {
-				clickit := false
-				app.ClickTarget(func(k tcell.ButtonMask, v gowid.IIdentityWidget) {
-					if v != nil && v.ID() == w.ID() {
-						clickit = true
-					}
-				})
-				if clickit {
-					curPosition := startPosition
-					saveState := w.st
+			if childSelectable {
+				// tcell will report ButtonNone for mouse events which are simply pointer movements
+				// (at least in my terminal). To distinguish this from a mouse release event, we track
+				// the prior input's mouse state. If the last state was a mouse click, then this event
+				// is processed as a mouse button release.
+				if !app.GetLastMouseState().NoButtonClicked() {
+					clickit := false
+					app.ClickTarget(func(k tcell.ButtonMask, v gowid.IIdentityWidget) {
+						if v != nil && v.ID() == w.ID() {
+							clickit = true
+						}
+					})
+					if clickit {
+						// This means the mouse button was released over widget w, after earlier having
+						// been clicked on widget w
+						curPosition := startPosition
+						saveState := w.st
 
-					for {
-						if curPosition.Equal(position) {
-							res = true
-							break
-						} else if dirMoved > 0 && curPosition.GreaterThan(position) {
-							res = false
-							w.st = saveState
-							w.Walker().SetFocus(startPosition, app)
-							break
-						} else if dirMoved < 0 && position.GreaterThan(curPosition) {
-							res = false
-							w.st = saveState
-							w.Walker().SetFocus(startPosition, app)
-							break
-						}
-						if dirMoved > 0 {
-							_, curPosition = w.MoveToNextFocus(subRenderSize, focus, numLinesToUse, app)
-						} else if dirMoved < 0 {
-							_, curPosition = w.MoveToPreviousFocus(subRenderSize, focus, numLinesToUse, app)
-						} else {
-							panic(BadState)
+						for {
+							if curPosition.Equal(position) {
+								res = true
+								break
+							} else if dirMoved > 0 && curPosition.GreaterThan(position) {
+								res = false
+								w.st = saveState
+								w.Walker().SetFocus(startPosition, app)
+								break
+							} else if dirMoved < 0 && position.GreaterThan(curPosition) {
+								res = false
+								w.st = saveState
+								w.Walker().SetFocus(startPosition, app)
+								break
+							}
+							if dirMoved > 0 {
+								_, curPosition = w.MoveToNextFocus(subRenderSize, focus, numLinesToUse, app)
+							} else if dirMoved < 0 {
+								_, curPosition = w.MoveToPreviousFocus(subRenderSize, focus, numLinesToUse, app)
+							} else {
+								panic(BadState)
+							}
 						}
+						//res = true
 					}
-					//res = true
 				}
 			}
 		}
@@ -1056,7 +1078,7 @@ func (w *Widget) MoveToNextFocus(subRenderSize gowid.IRenderSize, focus gowid.Se
 		return false, cur
 	}
 	oldPos := cur
-	curLines := gowid.RenderSize(curw, subRenderSize, gowid.NotSelected, app).BoxRows()
+	curLinesNoFocus := gowid.RenderSize(curw, subRenderSize, gowid.NotSelected, app).BoxRows()
 
 	// from that, get the next widget and next position. The nextw is used to run callbacks.
 	var next IWalkerPosition
@@ -1070,23 +1092,20 @@ func (w *Widget) MoveToNextFocus(subRenderSize gowid.IRenderSize, focus gowid.Se
 		if nextw.Selectable() {
 			break
 		}
-		curLines += gowid.RenderSize(nextw, subRenderSize, gowid.NotSelected, app).BoxRows()
+		curLinesNoFocus += gowid.RenderSize(nextw, subRenderSize, gowid.NotSelected, app).BoxRows()
 		cur = next
 	}
 
 	w.Walker().SetFocus(next, app)
-	if !next.Equal(oldPos) {
-		gowid.RunWidgetCallbacks(w, gowid.FocusCB{}, app, nextw)
-	}
 
 	nextLines := gowid.RenderSize(nextw, subRenderSize, focus, app).BoxRows()
 
 	// curWidgetLines has the number of lines used by the current focus widget when rendered. Compute how
-	// many lines should be above it, and how many below it.
+	// many line)s should be above it, and how many below it.
 	var computedLinesAbove, computedLinesBelow int
 	if !w.AtBottom() {
 		computedLinesAbove = gwutil.RoundFloatToInt(float32(gwutil.Max(0, screenLines)) * w.st.topToBottomRatio)
-		computedLinesAbove += curLines
+		computedLinesAbove += curLinesNoFocus
 		computedLinesBelow = screenLines - (computedLinesAbove + nextLines)
 		if computedLinesBelow <= 0 {
 			w.GoToBottom(app)
@@ -1095,6 +1114,12 @@ func (w *Widget) MoveToNextFocus(subRenderSize gowid.IRenderSize, focus gowid.Se
 			w.st.topToBottomRatio = float32(computedLinesAbove) / float32(screenLines)
 		}
 	}
+	w.st.linesOffTop = 0
+
+	// Do this at the end in case the focus callback wants to save the list state too.
+	if !next.Equal(oldPos) {
+		gowid.RunWidgetCallbacks(w, gowid.FocusCB{}, app, nextw)
+	}
 
 	return true, next
 }
@@ -1106,7 +1131,8 @@ func (w *Widget) MoveToPreviousFocus(subRenderSize gowid.IRenderSize, focus gowi
 	cur := w.Walker().Focus()
 	curw := w.Walker().At(cur)
 	oldpos := cur
-	curLines := gowid.RenderSize(curw, subRenderSize, focus, app).BoxRows()
+	curLinesFocus := gowid.RenderSize(curw, subRenderSize, focus, app).BoxRows()
+	betweenNoFocus := 0
 
 	// from that, get the next widget and next position. The nextw is used to run callbacks.
 	var prev IWalkerPosition
@@ -1120,33 +1146,40 @@ func (w *Widget) MoveToPreviousFocus(subRenderSize gowid.IRenderSize, focus gowi
 		if prevw.Selectable() {
 			break
 		}
-		curLines += gowid.RenderSize(prevw, subRenderSize, gowid.NotSelected, app).BoxRows()
+		betweenNoFocus += gowid.RenderSize(prevw, subRenderSize, gowid.NotSelected, app).BoxRows()
 		cur = prev
 	}
 
 	w.Walker().SetFocus(prev, app)
-	if !prev.Equal(oldpos) {
-		gowid.RunWidgetCallbacks(w, gowid.FocusCB{}, app, prevw)
-	}
 
-	prevLines := gowid.RenderSize(prevw, subRenderSize, gowid.NotSelected, app).BoxRows()
+	prevLinesNoFocus := gowid.RenderSize(prevw, subRenderSize, gowid.NotSelected, app).BoxRows()
 
 	// curWidgetLines has the number of lines used by the current focus widget when rendered. Compute how
 	// many lines should be above it, and how many below it.
 	var computedLinesAbove int
 	if wasAtBottom {
-		computedLinesAbove = gwutil.Max(0, screenLines) - curLines
+		computedLinesAbove = gwutil.Max(0, screenLines) - (curLinesFocus + betweenNoFocus + prevLinesNoFocus)
 	} else {
 		computedLinesAbove = gwutil.RoundFloatToInt(float32(gwutil.Max(0, screenLines)) * w.st.topToBottomRatio)
+		// Preserve lines *above* focus - it feels the most natural when scrolling. So if the
+		// previous widget (below) takes 3 lines to render with focus, but only 1 without, then add just
+		// one because that widget will only contribute 1 when it's no longer current.
+		computedLinesAbove -= (prevLinesNoFocus + betweenNoFocus)
 	}
-	computedLinesAbove -= prevLines
 	if computedLinesAbove <= 0 {
-		w.GoToTop(app)
+		prevLinesFocus := gowid.RenderSize(prevw, subRenderSize, focus, app).BoxRows()
+		w.GoToTop(app)                                               // widget is logically top, but might have lines cut if too big (see next line)
+		w.st.linesOffTop = gwutil.Max(0, prevLinesFocus-screenLines) // in case prev doesn't fit, start at bottom
 	} else {
 		w.st.topToBottomRatioValid = true
 		w.st.topToBottomRatio = float32(computedLinesAbove) / float32(screenLines)
 	}
 
+	// Do this at the end in case the focus callback wants to save the list state too.
+	if !prev.Equal(oldpos) {
+		gowid.RunWidgetCallbacks(w, gowid.FocusCB{}, app, prevw)
+	}
+
 	return true, prev
 }
 
