@@ -23,16 +23,18 @@ type IWidget interface {
 	gowid.IFindNextSelectable
 	gowid.IPreferedPosition
 	gowid.ISelectChild
+	gowid.IIdentity
 	RenderBoxMaker(size gowid.IRenderSize, focus gowid.Selector, focusIdx int, app gowid.IApp, fn IPileBoxMaker) ([]gowid.IRenderBox, []gowid.IRenderSize)
 	Wrap() bool
 }
 
 type Widget struct {
-	widgets   []gowid.IContainerWidget
-	focus     int // -1 means nothing selectable
-	prefRow   int // caches the last set prefered row. Passes it on if widget hasn't changed focus
-	opt       Options
-	Callbacks *gowid.Callbacks
+	widgets []gowid.IContainerWidget
+	focus   int // -1 means nothing selectable
+	prefRow int // caches the last set prefered row. Passes it on if widget hasn't changed focus
+	opt     Options
+	*gowid.Callbacks
+	gowid.AddressProvidesID
 	gowid.FocusCallbacks
 	gowid.SubWidgetsCallbacks
 }
@@ -58,16 +60,14 @@ func New(widgets []gowid.IContainerWidget, opts ...Options) *Widget {
 		}
 	}
 
-	cb := gowid.NewCallbacks()
 	res := &Widget{
-		widgets:             widgets,
-		focus:               -1,
-		prefRow:             -1,
-		opt:                 opt,
-		Callbacks:           cb,
-		FocusCallbacks:      gowid.FocusCallbacks{ICallbacks: cb},
-		SubWidgetsCallbacks: gowid.SubWidgetsCallbacks{cb},
+		widgets: widgets,
+		focus:   -1,
+		prefRow: -1,
+		opt:     opt,
 	}
+	res.FocusCallbacks = gowid.FocusCallbacks{CB: &res.Callbacks}
+	res.SubWidgetsCallbacks = gowid.SubWidgetsCallbacks{CB: &res.Callbacks}
 	if opt.StartRow >= 0 {
 		res.focus = gwutil.Min(opt.StartRow, len(widgets)-1)
 	} else {
@@ -259,11 +259,16 @@ func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Se
 	subs := w.SubWidgets()
 
 	focusEvent := func(selectable bool) {
+		srows := 0
+		for i := 0; i < subfocus; i++ {
+			srows += ss[i].BoxRows()
+		}
+
 		subSize := ss2[subfocus]
 		if selectable {
-			forChild = gowid.UserInput(subs[subfocus], gowid.TranslatedMouseEvent(ev, 0, -ss[subfocus].BoxRows()), subSize, focus, app)
+			forChild = subs[subfocus].UserInput(gowid.TranslatedMouseEvent(ev, 0, -srows), subSize, focus, app)
 		} else {
-			forChild = gowid.UserInputIfSelectable(subs[subfocus], gowid.TranslatedMouseEvent(ev, 0, -ss[subfocus].BoxRows()), subSize, focus, app)
+			forChild = gowid.UserInputIfSelectable(subs[subfocus], gowid.TranslatedMouseEvent(ev, 0, -srows), subSize, focus, app)
 		}
 	}
 
@@ -285,18 +290,39 @@ func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Se
 			if subfocus == -1 {
 				break
 			}
+
+			// A left click sets focus if the widget is selectable and would take the mouse input; but
+			// if I don't filter by click, then moving the mouse over another widget would shift focus
+			// automatically, which is not usually what's wanted.
 			_, my := evm.Position()
 			curY := 0
+		Loop:
 			for i, c := range ss {
 				if my < curY+c.BoxRows() && my >= curY {
 					subSize := ss2[i]
-					forChild = gowid.UserInput(subs[i], gowid.TranslatedMouseEvent(ev, 0, -curY), subSize, focus.SelectIf(w.SelectChild(focus) && i == subfocus), app)
-					// A left click always sets focus if the widget is selectable
-					//if forChild && ev2.Buttons()&tcell.Button1|tcell.WheelUp|tcell.WheelDown != 0 {
-					if forChild && evm.Buttons()&tcell.Button1 != 0 {
-						w.SetFocus(app, i)
+					forChild = subs[i].UserInput(gowid.TranslatedMouseEvent(ev, 0, -curY), subSize, focus.SelectIf(w.SelectChild(focus) && i == subfocus), app)
+
+					// Give the child focus if (a) it's selectable, and (b) if this is the click up corresponding
+					// to a previous click down on this pile widget.
+					switch evm.Buttons() {
+					case tcell.Button1, tcell.Button2, tcell.Button3:
+						app.SetClickTarget(evm.Buttons(), w)
+					case tcell.ButtonNone:
+						if !app.GetLastMouseState().NoButtonClicked() {
+							if subs[i].Selectable() {
+								clickit := false
+								app.ClickTarget(func(k tcell.ButtonMask, v gowid.IIdentityWidget) {
+									if v != nil && v.ID() == w.ID() {
+										clickit = true
+									}
+								})
+								if clickit {
+									w.SetFocus(app, i)
+								}
+							}
+						}
+						break Loop
 					}
-					break
 				}
 				curY += c.BoxRows()
 			}
@@ -448,7 +474,7 @@ func RenderedChildrenSizes(w IWidget, size gowid.IRenderSize, focus gowid.Select
 
 func RenderSubwidgets(w IWidget, size gowid.IRenderSize, focus gowid.Selector, focusIdx int, app gowid.IApp) []gowid.ICanvas {
 	fn1 := BoxMakerFunc(func(w gowid.IWidget, subSize gowid.IRenderSize, focus gowid.Selector, subApp gowid.IApp) gowid.IRenderBox {
-		return gowid.Render(w, subSize, focus, subApp)
+		return w.Render(subSize, focus, subApp)
 	})
 
 	canvases, _ := w.RenderBoxMaker(size, focus, focusIdx, app, fn1)
