@@ -15,6 +15,7 @@ import (
 	"github.com/gcla/gowid/widgets/isselected"
 	"github.com/gcla/gowid/widgets/list"
 	"github.com/gcla/gowid/widgets/pile"
+	"github.com/gdamore/tcell"
 	lru "github.com/hashicorp/golang-lru"
 )
 
@@ -31,7 +32,7 @@ import (
 // cache.
 type RowId int
 
-// ITable is implemented by any type which can provide arrays of
+// IModel is implemented by any type which can provide arrays of
 // widgets for a given table row, and optionally header widgets.
 type IModel interface {
 	Columns() int
@@ -44,6 +45,10 @@ type IModel interface {
 	Widths() []gowid.IWidgetDimension
 }
 
+type IInvertible interface {
+	IdentifierToRow(rowid RowId) (int, bool)
+}
+
 type IMakeHeader interface {
 	HeaderWidget([]gowid.IWidget, int) gowid.IWidget
 }
@@ -195,7 +200,7 @@ type Widget struct {
 	flowVertDivider  *gowid.ContainerWidget
 	flowTableDivider *gowid.ContainerWidget
 	opt              Options
-	Callbacks        *gowid.Callbacks
+	*gowid.Callbacks
 	gowid.FocusCallbacks
 	gowid.IsSelectable
 }
@@ -220,7 +225,6 @@ func New(model IModel, opts ...Options) *Widget {
 		opt = opts[0]
 	}
 
-	cb := gowid.NewCallbacks()
 	// Fill in Widget later once constructed.
 	listw := &ListWithPreferedColumn{}
 
@@ -240,13 +244,13 @@ func New(model IModel, opts ...Options) *Widget {
 
 	// res acts as a ListWalker and a widget
 	res := &Widget{
-		listw:          listw,
-		cur:            0,
-		cache:          cache,
-		Callbacks:      cb,
-		FocusCallbacks: gowid.FocusCallbacks{ICallbacks: cb},
+		listw: listw,
+		cur:   0,
+		cache: cache,
 	}
 
+	res.FocusCallbacks = gowid.FocusCallbacks{CB: &res.Callbacks}
+
 	switch model.(type) {
 	case IBoundedModel:
 		listw.IWidget = list.NewBounded(&BoundedWidget{res})
@@ -390,6 +394,14 @@ func (w *BoundedWidget) Last() list.IWalkerPosition {
 	}
 }
 
+func (w *BoundedWidget) BoundedWalker() list.IBoundedWalker {
+	return w.listw.Walker().(list.IBoundedWalker)
+}
+
+func (w *BoundedWidget) Pos() int {
+	return w.listw.Walker().(list.IBoundedWalker).Focus().(list.IBoundedWalkerPosition).ToInt()
+}
+
 func (w *BoundedWidget) Length() int {
 	//return w.listw.IWidget.(*list.IndexedWidget).Walker().(list.IBoundedWalker).Length()
 	return w.Model().(IBoundedModel).Rows()
@@ -401,6 +413,7 @@ func (w *Widget) CalculateOnScreen(size gowid.IRenderSize, focus gowid.Selector,
 
 func (w *Widget) SetModel(model IModel, app gowid.IApp) {
 	oldpos, olderr := w.FocusXY()
+	w.cache.Purge() // gcla later todo
 	w.update(w.listw, w.cur, model, w.opt)
 	if olderr == nil {
 		w.SetFocusXY(app, oldpos) // mght not be able to set old focus, if model shape has changed
@@ -487,9 +500,20 @@ func (w *Widget) RenderSize(size gowid.IRenderSize, focus gowid.Selector, app go
 	return gowid.CalculateRenderSizeFallback(w, size, focus, app)
 }
 
+func (w *Widget) SetFocusOnHeader(app gowid.IApp) {
+	w.wrapper.SetFocus(app, 0)
+}
+
+// SetFocusOnData returns true if there is data to focus on
+func (w *Widget) SetFocusOnData(app gowid.IApp) bool {
+	cur := w.wrapper.Focus()
+	w.wrapper.SetFocus(app, 1)
+	return cur != w.wrapper.Focus()
+}
+
 func (w *Widget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) bool {
 	oldpos, olderr := w.FocusXY()
-	res := gowid.UserInput(w.wrapper, ev, size, focus, app)
+	res := w.wrapper.UserInput(ev, size, focus, app)
 	newpos, newerr := w.FocusXY()
 	if olderr != newerr || oldpos != newpos {
 		gowid.RunWidgetCallbacks(w.Callbacks, gowid.FocusCB{}, app, w)
@@ -501,6 +525,30 @@ func (w *Widget) Render(size gowid.IRenderSize, focus gowid.Selector, app gowid.
 	return w.wrapper.Render(size, focus, app)
 }
 
+func (w *Widget) Up(lines int, size gowid.IRenderSize, app gowid.IApp) {
+	for i := 0; i < lines; i++ {
+		w.wrapper.UserInput(tcell.NewEventKey(tcell.KeyUp, ' ', tcell.ModNone), size, gowid.Focused, app)
+	}
+}
+
+func (w *Widget) Down(lines int, size gowid.IRenderSize, app gowid.IApp) {
+	for i := 0; i < lines; i++ {
+		w.wrapper.UserInput(tcell.NewEventKey(tcell.KeyDown, ' ', tcell.ModNone), size, gowid.Focused, app)
+	}
+}
+
+func (w *Widget) UpPage(num int, size gowid.IRenderSize, app gowid.IApp) {
+	for i := 0; i < num; i++ {
+		w.wrapper.UserInput(tcell.NewEventKey(tcell.KeyPgUp, ' ', tcell.ModNone), size, gowid.Focused, app)
+	}
+}
+
+func (w *Widget) DownPage(num int, size gowid.IRenderSize, app gowid.IApp) {
+	for i := 0; i < num; i++ {
+		w.wrapper.UserInput(tcell.NewEventKey(tcell.KeyPgDn, ' ', tcell.ModNone), size, gowid.Focused, app)
+	}
+}
+
 type IRowToWidget interface {
 	VertDivider() gowid.IContainerWidget
 	Model() IModel
@@ -697,6 +745,30 @@ func (t *Widget) Previous(ipos list.IWalkerPosition) list.IWalkerPosition {
 	}
 }
 
+type IGoToBottom interface {
+	GoToBottom(app gowid.IApp)
+}
+
+func (t *Widget) GoToBottom(app gowid.IApp) bool {
+	if b, ok := t.listw.IWidget.(IGoToBottom); ok {
+		b.GoToBottom(app)
+		return true
+	}
+	return false
+}
+
+type IGoToMiddle interface {
+	GoToMiddle(app gowid.IApp)
+}
+
+func (t *Widget) GoToMiddle(app gowid.IApp) bool {
+	if b, ok := t.listw.IWidget.(IGoToMiddle); ok {
+		b.GoToMiddle(app)
+		return true
+	}
+	return false
+}
+
 type Coords struct {
 	Column int
 	Row    int
